'/* TESTTEXT.BAS  Check text/pointer files for proper index sequence */
'/*               By: Dale Thorn                                     */
'/*               Rev. 16.10.1997                                    */

'$include: 'basdef.h'
'$include: 'filekill.h'
'$include: 'fileopen.h'
'$include: 'messages.h'
'$include: 'midchar.h'
'$include: 'parmstr1.h'
'$include: 'basdef.bas'
'$include: 'filekill.bas'
'$include: 'fileopen.bas'
'$include: 'messages.bas'
'$include: 'midchar.bas'
'$include: 'parmstr1.bas'
'$include: 'scrnparm.bas'

dim ccur(50), cprv(50)                'string comparison (prev./current) arrays

ccmd = ucase$(rtrim$(command$))            'get the DOS command-line parameters

if ccmd = "" then                              'a command line was NOT supplied
   i = ifn.msgs("Usage:  TESTTEXT  filename  [beginindex]", iofs, irow, icol, 0, 1)
end if                              'display the usage message [above] and exit

iprm = parmstr1(ccmd, cfil, cnam, cext, cprm())  'parse command-line parameters

if cnam = "" or len(cnam) > 8 or len(cext) > 3 or instr(cext, ".") then
   i = ifn.msgs("Invalid filename", iofs, irow, icol, 1, 1)   'invalid filename
end if                       'display invalid-filename message [above] and exit

if iprm > 0 then                    'number of parameters should be zero or one
   i = ifn.msgs("Invalid number of parameters", iofs, irow, icol, 1, 1)
end if                      'display no.-of-parameters message [above] and exit

if iprm = 0 then               'User input a pointer-file begin-"record" number
   lbeg = pdqvall&(cprm(0))         'get the pointer-file begin-"record" number
   if lbeg < 1 or lbeg > 16777215 or ltrim$(str$(lbeg)) <> cprm(0) then
      i = ifn.msgs("Invalid beginindex parameter", iofs, irow, icol, 1, 1)
   end if                   'display invalid-parameter message [above] and exit
else                   'User did NOT input a pointer-file begin-"record" number
   lbeg = 1                 'set the default pointer-file begin-"record" number
end if

i = ifn.open(1, cfil, "B", lof1)             'open the text file in binary mode
if lof1 < 0 then                                'user input a wildcard filespec
   i = ifn.msgs("Invalid filename", iofs, irow, icol, 1, 1)      'beep and exit
elseif lof1 = 0 then                            'text file nonexistent or empty
   i = ifn.kill(1, cfil)                             'kill the zero-length file
   i = ifn.msgs(cfil + " not found", iofs, irow, icol, 1, 1)     'beep and exit
end if

i = ifn.open(2, cnam + ".ptr", "B", lof2)   'open the index file in binary mode
if lof2 = 0 then                               'index file nonexistent or empty
   i = ifn.kill(2, cnam + ".ptr")                    'kill the zero-length file
   i = ifn.msgs(cnam + ".PTR not found", iofs, irow, icol, 1, 1) 'beep and exit
end if                         'display file-not-found message [above] and exit
lof2 = lof2 - 4                       'set working length of index pointer file

cwrd = space$(2)                   'allocate a temporary buffer for word length
get 2, lof2 + 3, cwrd                'get the maximum-word-length specification
iwrd = cvi(cwrd)                     'set the maximum-word-length specification

i = ifn.open(3, "wmak.wrd", "B", lof3)   'open valid-chars. file in binary mode
if lof3 = 0 then                        'valid-chars. file nonexistent or empty
   i = ifn.kill(3, "wmak.wrd")                       'kill the zero-length file
   i = ifn.msgs("WMAK.WRD not found", iofs, irow, icol, 1, 1)    'beep and exit
end if                         'display file-not-found message [above] and exit

cwrd = space$(lof3)                 'allocate valid-word-characters file buffer
get 3, clng(1), cwrd                      'get the valid-word-characters string
close 3                                   'close the valid-word-characters file
for i = lof3 to 1 step -1           'loop f/end of valid-word characters string
   if midchar(cwrd, i) >= 33 then                 'last valid character in file
      exit for                           'valid-word string verified; exit loop
   end if
next
if i < 2 then        'valid-word characters NOT found; display message and exit
   i = ifn.msgs("Invalid WMAK.WRD file", iofs, irow, icol, 1, 1)  'invalid file
else                                  'valid-word characters were found in file
   cwrd = left$(cwrd, i)                   'preserve valid-word characters only
end if

i = ifn.msgs("Please standby", iofs, irow, icol, 0, 0)

for i = 1 to iwrd                              'loop to the maximum word length
   cprv(i) = space$(i)  'allocate "previous" buffer to this partial word length
   ccur(i) = space$(i)   'allocate "current" buffer to this partial word length
next

cbuf = space$(iwrd)                'buffer to load a word of up to maximum size
cndx = space$(3072)               'buffer to load 1024 index pointers at a time
cptr = string$(4, 0)                'string form of one index pointer [for CVL]
lmax = lof2 - 3071                    'maximum load-from position in index file

cmsg = "Index test of " + cfil + " completed"    'initialize completion message
iprv = 1                                    'initialize length of previous word
ibrp = 1                                            'initialize user-alert beep
for lrcd = lbeg * 3 - 2 to lof2 step 3072   'walk index file in 3072-byte steps
   if lrcd > lmax then                  'last buffer in index file < 3072 bytes
      cndx = ""                           'deallocate index pointer file buffer
      cndx = space$(lof2 - lrcd + 1)      'reallocate index pointer file buffer
   end if
   get 2, lrcd, cndx                        'load 1024 pointers from index file
   for indx = 1 to len(cndx) step 3          'walk index buffer in 3-byte steps
      mid$(cptr, 1) = char(midchar(cndx, indx + 0)) 'put 1st byte to CVL buffer
      mid$(cptr, 2) = char(midchar(cndx, indx + 1)) 'put 2nd byte to CVL buffer
      mid$(cptr, 3) = char(midchar(cndx, indx + 2)) 'put 3rd byte to CVL buffer
      lptr = cvl(cptr) + 1        'convert zero-base pointer to BASIC (+1) form
      get 1, lptr, cbuf            'load word pointed to by lptr from text file
      if lptr + iwrd - 1 > lof1 then 'word buffer was loaded to beyond text EOF
         mid$(cbuf, lof1 - lptr + 2) = space$(iwrd)    'blank buffer beyond EOF
      end if
      for ipos = 1 to iwrd                     'loop to the maximum word length
         if instr(cwrd, char(midchar(cbuf, ipos))) = 0 then 'non-word character
            exit for             'exit loop when non-valid word character found
         end if
      next
      ilen = ipos - 1                                   'length of current word
      lset ccur(ilen) = ucase$(cbuf)  'put uppercased word to exact-size buffer
      if ccur(ilen) < cprv(iprv) then             'current word < previous word
         cmsg = "Bad data:  " + cprv(iprv) + "  " + ccur(ilen)'bad data message
         lrcd = lof2                   'set outer-loop index to exit outer loop
         exit for          'exit inner loop [and automatically exit outer loop]
      end if
      if iprm = 0 then                  'user specified a begin-index parameter
         print (lrcd + indx + 1) \ 3; "  "; cprv(iprv); "  "; ccur(ilen)
         do       'display prev. & curr. words [above] & wait for user response
            ckey = inkey$                      'get next user keypress [if any]
         loop while ckey = ""                    'loop until user presses a key
      else                        'user did NOT specify a begin-index parameter
         ckey = inkey$                              'get user keypress [if any]
      end if
      lset cprv(ilen) = ccur(ilen)    'put current word to previous-word buffer
      iprv = ilen                'set exact-size buffer index for previous word
      if ckey = char(27) then                     'user pressed ESC key [above]
         cmsg = "Abort selected - Program terminated"   'set user-abort message
         lrcd = lof2                   'set outer-loop index to exit outer loop
         ibrp = 0                                     'turn user-alert beep OFF
         exit for          'exit inner loop [and automatically exit outer loop]
      end if
   next
next

i = ifn.msgs(cmsg, iofs, irow, icol, ibrp, 1) 'display message set above & exit

close                                 'close all files in case not closed above
system                                                'exit to operating system
