/* WMAK.C  Process "words" in an ASCII text file (output to 2nd file) */
/*         By: Dale Thorn                                             */
/*         Rev. 28.02.2001                                            */

#include "stddef.h"
#include "stdlib.h"
#include "string.h"
#include "stdio.h"
#include "cbasic.h"

V strparm(const C *cstr, I *ibeg, I *ilen);

I imrg = 4;                            /* column offset for display messages */

V main(I argc, C **argv) {     /* command-line arguments (input file/offset) */
   C cbuf[600];                               /* input-file line data buffer */
   C ccmp[128];                                       /* word-compare buffer */
   C *cexc;                               /* pointer to excluded-word buffer */
   C *cinp;                              /* pointer to word-position in cbuf */
   C cmax[128];                       /* longest word found in the text file */
   C *cmrg = "    ";                   /* column offset for display messages */
   C cnam[128];                      /* file name/path or word-output buffer */
   C cwrd[128];                                 /* in-word characters buffer */
   I ibeg = 0;                                    /* beginning-of-word value */
   I iend = 0;                                  /* end-of-word flag or value */
   I iexc = 0;                        /* maximum length of an exclusion word */
   I iext = 0;                  /* maximum word length plus extension-length */
   I ilen = 0;                                 /* raw length of current word */
   I imax = 0;                      /* maximum length of all words processed */
   I imwl = 124;                            /* maximum allowable word length */
   I inet;                                     /* net length of current word */
   I iofs = 0;                               /* begin parsing at this offset */
   I iseq = 0;                 /* process sequence 1=get imax;  2=read&write */
   I itot = 0;                        /* initialize the exclusion-word total */
   L llin = 0;                     /* current input-file line-begin position */
   L lpos = 0;                    /* current output-file data-write position */
   L ltot = 0;                                  /* initialize the word total */
   L lwrd;                         /* current input-file word-begin position */
   struct _iobuf *ebuf;                               /* .ERR FILE structure */
   struct _iobuf *ibuf;                              /* input FILE structure */
   struct _iobuf *pbuf;                               /* .PTS FILE structure */
   struct _iobuf *wbuf;                               /* .WRD FILE structure */
   struct _iobuf *xbuf;                               /* .EXC FILE structure */

   ebuf = fopen("wmak.err", "wb");                  /* open a new error file */
   fseek(ebuf, 0, SEEK_SET);                         /* error-write position */
   fwrite((V *)"9", 1, 1, ebuf);         /* prewrite an "undetermined" error */
   fseek(ebuf, 0, SEEK_SET);                         /* error-write position */
   cmrg[imrg] = '\0';                  /* column offset for display messages */

   if ((ibuf = fopen(argv[1], "r")) == NULL) {        /* open the input file */
      fwrite((V *)"1", 1, 1, ebuf);                       /* write the error */
      fclose(ebuf);                                  /* close the error file */
      if (argc == 1) {      /* a file was NOT specified; display msg. & exit */
         printf("\n%s%s\n", cmrg, "Usage:  WMAK  filename  [offset]");
      } else {                         /* a file was specified but not found */
         printf("\n%s%s%s\n", cmrg, argv[1], " not found");/* message & exit */
         printf("\a");                          /* sound the user-alert tone */
      }
      exit(0);                           /* return to DOS or calling program */
   }

   if ((xbuf = fopen("wmak.exc", "r")) != NULL) {      /* open the .EXC file */
      while (fgets(cbuf, 128, xbuf) != NULL) { /* loop till exclusions = EOF */
         strparm(cbuf, &ibeg, &ilen);        /* get begin and length of word */
         if (ilen) {               /* this is a valid word insofar as length */
            itot++;                    /* increment the exclusion-word total */
            if (iexc < ilen) {     /* current word longer than prev. maximum */
               iexc = ilen;           /* set maximum length = current length */
            }
         }
      }
      fclose(xbuf);                                   /* close the .EXC file */
      iexc++;                   /* set iexc to word-segment length in buffer */
   }
   if (itot) {                          /* at least one valid word was found */
      if ((cexc = (C *)malloc(30000)) == NULL) {  /* allocate a "30k" buffer */
         fwrite((V *)"3", 1, 1, ebuf);                    /* write the error */
         fclose(ebuf);                               /* close the error file */
         fclose(ibuf);                               /* close the input file */
         printf("\n%s%s\n", cmrg, "Memory allocation failed");/* msg. & exit */
         printf("\a");                          /* sound the user-alert tone */
         exit(0);                /* exit if failure; use error file as above */
      }
      memset(cexc, ' ', iexc * itot);          /* set buffer to blank spaces */
      cexc[iexc * itot] = '\0';                      /* terminate the buffer */
      xbuf = fopen("wmak.exc", "r");                 /* reopen the .EXC file */
      cinp = cexc;                        /* begin filling buffer at byte #0 */
      while (fgets(cbuf, 128, xbuf) != NULL) { /* loop till exclusions = EOF */
         strparm(cbuf, &ibeg, &ilen);        /* get begin and length of word */
         if (ilen) {               /* this is a valid word insofar as length */
            strupr(cbuf + ibeg);                       /* uppercase the word */
            memcpy(cinp, cbuf + ibeg, ilen); /* put the word into the buffer */
            cinp += iexc;             /* increment the buffer write-position */
         }
      }
      fclose(xbuf);                                   /* close the .EXC file */
   } else {                         /* no valid words or .EXC file not found */
      cexc = (C *)calloc(1, 1);                    /* allocate a "0k" buffer */
   }

   if ((wbuf = fopen("wmak.wrd", "r")) != NULL) {      /* open the .WRD file */
      fgets(cwrd, 128, wbuf);                      /* get the in-word string */
      cwrd[strcspn(cwrd, "\n")] = '\0';        /* eliminate the '\n' in cwrd */
      fclose(wbuf);                                   /* close the .WRD file */
      iend = strlen(cwrd);                         /* get the length of cwrd */
      ilen = iend;                                        /* set ilen = iend */
      while ((ibeg = strcspn(cwrd, " ")) < ilen) { /* check for blank spaces */
         memmove(cwrd + ibeg, cwrd + ibeg + 1, ilen - ibeg);   /* shift cwrd */
         ilen--;                             /* decrement the length of cwrd */
      }
      if (ilen < iend) {              /* blank spaces were found in WMAK.WRD */
         remove("wmak.wrd");                /* attempt to kill the .WRD file */
         wbuf = fopen("wmak.wrd", "w");              /* reopen the .WRD file */
         fputs(cwrd, wbuf);                      /* write the in-word string */
         fputs("\n", wbuf);                          /* write the terminator */
         fclose(wbuf);                                /* close the .WRD file */
      }
   } else {                              /* an in-word file was not supplied */
      strcpy(cwrd, "$%'0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ");    /* default */
      strcat(cwrd, "abcdefghijklmnopqrstuvwxyz");      /* in-word characters */
   }
   iend = 0;                              /* reset end-of-word flag or value */
   ilen = 0;                        /* reset ilen for word-length processing */

   strcpy(cnam, argv[1]);                  /* copy input name to output name */
   strcpy(cnam + strcspn(cnam, "."), ".pts");    /* new extension for output */
   remove(cnam);                     /* attempt to kill existing output file */
   pbuf = fopen(cnam, "wb");                       /* open a new output file */

   for (iseq = 0; iseq <= 1; iseq++) {        /* loop thru process sequences */
      if (iseq) {                              /* 2nd sequence: read & write */
         ibuf = fopen(argv[1], "r");                /* reopen the input file */
         iext = imax + (argc >= 3) * 4;               /* set extended-length */
      } else {                              /* 1st sequence: get imax length */
         if (argc >= 3) {                   /* an offset value was specified */
            iofs = atoi(argv[2] + strspn(argv[2], "/")); /* set offset value */
         }
         if (argc >= 4) {             /* a maximum word length was specified */
            imwl = atoi(argv[3] + strspn(argv[3], "/")); /* set max. wordlen */
         }
      }
      while (fgets(cbuf, 600, ibuf) != NULL) {      /* loop till input = EOF */
         for (ibeg = 0; ibeg < iofs; ibeg++) {   /* loop thru offset in cbuf */
            if (cbuf[ibeg] == '\0') {    /* terminator found in offset bytes */
               break;                           /* exit search at terminator */
            }
         }
         cinp = cbuf + ibeg;               /* begin parsing at cbuf + offset */
         cinp += strspn(cinp, " ");         /* beginning of 1st word in cbuf */
         while (*cinp != '\n' && *cinp != '\0') {  /* loop until end-of-line */
            if (strchr(cwrd, *cinp) != NULL) {       /* in-word character OK */
               ilen++;                          /* increment the word-length */
            } else {                             /* in-word character NOT OK */
               iend = 1;                               /* set end-of-word ON */
            }
            cinp++;                     /* increment buffer-position pointer */
            if (*cinp == '\n' || *cinp == '\0' || ilen == 124 || iend) {
               if (ilen) {              /* an actual word has been completed */
                  if (ilen > imwl) { /* current wordlen is > maximum wordlen */
                     inet = imwl;      /* set net wordlen to maximum wordlen */
                  } else {          /* current wordlen is <= maximum wordlen */
                     inet = ilen;      /* set net wordlen to current wordlen */
                  }
                  if (iseq) {           /* current sequence=WRITE; lset word */
                     cnam[0] = (UC)inet;      /* put length of word in front */
                     memcpy(cnam + 1, cinp - ilen - iend, inet);/* copy word */
                     cnam[inet + 1] = '\0';     /* termination for uppercase */
                     strupr(cnam + 1);                 /* uppercase the name */
                     strcpy(ccmp, cnam + 1);       /* copy to compare-buffer */
                     if (inet < iexc) {      /* word length < compare-length */
                        memset(ccmp + inet, ' ', iexc - inet);/* fill-spaces */
                     }
                     ccmp[iexc] = '\0';     /* temp. termination for compare */
                     if (strstr(cexc, ccmp) == NULL || !iexc) {/* no exclusn */
                        if (argc >= 3) {       /* offset value was specified */
                           lwrd = llin + (cinp - cbuf - ilen - iend);
                           memcpy(cnam + inet + 1, (C *)&lwrd, 3);
                        }         /* put word-pos. value into output [above] */
                        fseek(pbuf, lpos, SEEK_SET);  /* set position; write */
                        fwrite((V *)cnam, 1, (U)(inet + 4), pbuf); /* ..data */
                        lpos += inet + 4;        /* save next write-position */
                        ltot = ltot + 1;         /* increment the word total */
                     }
                  } else {                      /* current sequence=set imax */
                     if (imax < inet) {        /* current word > current max */
                        imax = inet;            /* reset maximum word-length */
                        memcpy(cmax, cinp - ilen - iend, inet); /* max. word */
                        cmax[inet] = '\0';        /* terminate the max. word */
                     }
                  }
               }
               iend = 0;                           /* reset end-of-word flag */
               ilen = 0;                        /* reset current-word length */
            }
            if (cinp - cbuf == 124 + 264) {      /* check for NO end-of-line */
               if (strchr(cinp, '\n') == NULL) {    /* end-of-line NOT found */
                  memmove(cbuf, cbuf + 265, 265);   /* shift the line buffer */
                  fgets(cbuf + 264, 266, ibuf);   /* get more text this line */
                  cinp -= 265;              /* shift the line-buffer pointer */
               }
            }
         }
         if (iseq) {                        /* current sequence = write data */
            llin += strlen(cbuf) + 1;       /* increment line-begin position */
         }
      }
      fclose(ibuf);                                  /* close the input file */
   }
   if (lpos) {                            /* at least one word was processed */
      fwrite((V *)"0", 1, 1, ebuf);                      /* write zero error */
      fseek(pbuf, lpos, SEEK_SET);           /* record-length write-position */
      fwrite((V *)&imax, 2, 1, pbuf);          /* write the max. word length */
      fseek(pbuf, lpos + 2, SEEK_SET);          /* word-total write-position */
      fwrite((V *)&ltot, 4, 1, pbuf);        /* write the total no. of words */
      fclose(ebuf);                                  /* close the error file */
      fclose(pbuf);                                 /* close the output file */
      printf("\n%s%s%ld\n", cmrg, "Total word count is: ", ltot);/* word-tot */
      printf("%s%s%s\n", cmrg, "The longest word is: ", cmax); /* disp. cmax */
      printf("%s%s%d\n", cmrg, "Longest word length: ", strlen(cmax));
   } else {            /* no valid words were found; display message [below] */
      fwrite((V *)"2", 1, 1, ebuf);        /* write a "no words found" error */
      fclose(ebuf);                                  /* close the error file */
      fclose(pbuf);                                 /* close the output file */
      printf("\n%s%s%s\n", cmrg, "No valid words were found in ", argv[1]);
   }                      /* display no-valid-words message [above] and exit */
   printf("\a");                                /* sound the user-alert tone */
}

V strparm(const C *cstr, I *ibeg, I *ilen) { /* get begin and length of word */
   *ibeg = strspn(cstr, " ");              /* begin search after first blank */
   *ilen = strcspn(cstr, "\n") - 1;         /* find position of '\n' or '\0' */
   while (*ibeg < *ilen) {                /* check for != ichr at ipos index */
      if (cstr[*ilen] == ' ') {                    /* match at this position */
         *ilen--;                          /* decrement to previous position */
      } else {                                  /* no match at this position */
         break;                                /* exit loop at this position */
      }
   }
   *ilen = *ilen - *ibeg + 1;            /* return length to calling program */
}
